@@ -34,7 +34,7 @@ application status
 ------------------
 
 Any application- and user-specific configurations like e.g. the last
-window position/size, the app theme/font or the last selected flow
+window position/size, the app theme/font/language or the last selected flow
 within your app, could be included in the application status.
 
 This namespace portion introduces the section `aeAppState` in the app
@@ -54,6 +54,7 @@ This module is providing/pre-defining the following application state variables:
     * :attr:`~MainAppBase.flow_id`
     * :attr:`~MainAppBase.flow_path`
     * :attr:`~MainAppBase.font_size`
+    * :attr:`~MainAppBase.lang_code`
     * :attr:`~MainAppBase.light_theme`
     * :attr:`~MainAppBase.sound_volume`
     * :attr:`~MainAppBase.win_rectangle`
@@ -377,14 +378,15 @@ import traceback
 from copy import deepcopy
 from typing import Any, Callable, Dict, Tuple, List, Optional, Type, Sequence
 
-from ae.core import stack_var                       # type: ignore
+from ae.core import DEBUG_LEVELS, stack_var         # type: ignore
 from ae.updater import check_all                    # type: ignore
 from ae.console import ConsoleApp                   # type: ignore
 
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
+from ae.i18n import default_language                # type: ignore
 
 
-__version__ = '0.0.25'
+__version__ = '0.0.28'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -520,6 +522,7 @@ class MainAppBase(ConsoleApp, ABC):
     flow_id: str = ""                                       #: id of the current app flow (entered by the app user)
     flow_path: List[str]                                    #: list of flow ids, reflecting recent user actions
     font_size: float = 30.0                                 #: font size used for toolbar and flow screens
+    lang_code: str = ""                                     #: optional language code (e.g. 'es_ES' for Spanish)
     light_theme: bool = False                               #: True=light theme/background, False=dark theme
     sound_volume: float = 0.12                              #: sound volume of current app (0.0=mute, 1.0=max)
     win_rectangle: tuple = (0, 0, 800, 600)                 #: window coordinates (x, y, width, height)
@@ -538,7 +541,7 @@ class MainAppBase(ConsoleApp, ABC):
         :param console_app_kwargs:  kwargs to be passed to the __init__ method of :class:`~ae.console_app.ConsoleApp`.
         """
         self._exit_code: int = 0                            #: init by stop_app() and passed onto OS by run_app()
-        self._last_focus_flow_id: str = id_of_flow('', '')  #: id of the last valid focused node item
+        self._last_focus_flow_id: str = id_of_flow('', '')  #: id of the last valid focused window/widget/item/context
 
         self._start_event_loop: Optional[Callable]          #: callable to start event loop of GUI framework
         self._stop_event_loop: Optional[Callable]           #: callable to start event loop of GUI framework
@@ -575,7 +578,7 @@ class MainAppBase(ConsoleApp, ABC):
         return self.cfg_section_variable_names(APP_STATE_SECTION_NAME)
 
     def change_app_state(self, state_name: str, new_value: Any, send_event: bool = True):
-        """ change single app state item to value in self.attribute and app_state dict item.
+        """ change single app state item to new_value in self.attribute and app_state dict item.
 
         :param state_name:  name of the app state to change.
         :param new_value:   new value of the app state to change.
@@ -609,6 +612,18 @@ class MainAppBase(ConsoleApp, ABC):
 
         self.setup_app_states(app_state)
 
+    def on_debug_level_change(self, debug_level: str, _event_kwargs: Dict[str, Any]) -> bool:
+        """ language app state change event handler.
+
+        :param debug_level:     the new debug level name to be set (passed as flow key).
+        :param _event_kwargs:   unused event kwargs.
+        :return:                True for to confirm the debug level change.
+        """
+        level = next(num for num, name in DEBUG_LEVELS.items() if name == debug_level)
+        self.vpo(f"MainAppBase.on_debug_level_change to {debug_level} -> {level}")
+        self.set_opt('debug_level', level)
+        return True
+
     def on_font_size_change(self, _flow_key: str, event_kwargs: Dict[str, Any]) -> bool:
         """ font size app state change event handler.
 
@@ -621,8 +636,22 @@ class MainAppBase(ConsoleApp, ABC):
         self.change_app_state('font_size', font_size)
         return True
 
+    def on_lang_code_change(self, lang_code: str, _event_kwargs: Dict[str, Any]) -> bool:
+        """ language app state change event handler.
+
+        :param lang_code:       the new language code to be set (passed as flow key).
+        :param _event_kwargs:   unused event kwargs.
+        :return:                True for to confirm the language change.
+        """
+        self.vpo(f"MainAppBase.on_lang_code_change to {lang_code}")
+        self.change_app_state('lang_code', lang_code)
+        if lang_code:
+            default_language(lang_code)
+            self.set_var('lang_code', lang_code, section=APP_STATE_SECTION_NAME)  # add optional app state var to config
+        return True
+
     def on_light_theme_change(self, _flow_key: str, event_kwargs: Dict[str, Any]) -> bool:
-        """ font size app state change event handler.
+        """ app theme app state change event handler.
 
         :param _flow_key:       flow key.
         :param event_kwargs:    event kwargs with key `'light_theme'` containing True|False for light|dark theme.
@@ -684,7 +713,7 @@ class MainAppBase(ConsoleApp, ABC):
         """
         self.vpo(f"MainAppBase.setup_app_states {app_state}")
         for key, val in app_state.items():
-            self.change_app_state(key, val, send_event=False)
+            self.change_app_state(key, val, send_event=False)   # don't send on_-events as framework is not initialized
             if key == 'flow_id' and flow_action(val) == 'focus':
                 self._last_focus_flow_id = val
 
@@ -900,7 +929,8 @@ class MainAppBase(ConsoleApp, ABC):
         en_lower = event_name.lower()
         if not self.call_method(event_name):
             if event_name == en_lower or not self.call_method(en_lower):
-                return self.call_method('on_key_press', modifiers, key)
+                # always pass lower key code (enaml/Qt is passing upper-case key code if Shift modifier is pressed)
+                return self.call_method('on_key_press', modifiers, key.lower())
         return True
 
     def load_images(self):
